/***************************
 * dicom dump by Tony Voet *
 ***************************/

#include <stdio.h>
#include <stdlib.h>
#include <dicom.h>

static void dump_print(const ELEMENT *,const char *);
static void dump_open(const char *);

static int verbose=0;

/********
 * main *
 ********/

int main(int argc,char *argv[])
{
  int i;

  if (argc<2)
  {
    fprintf(stderr,"%s (%s)\n",*argv,dicom_version);
    fprintf(stderr,"Usage: %s [-v] dicom-file ...\n",*argv);
    fprintf(stderr,"   -v: verbose output\n");

    return -1;
  }

  dicom_log_name(*argv);

  for (i=1; i<argc; i++)
    if (!strcmp(argv[i],"-v"))
      verbose=-1;
    else
    {
      printf("\n%s\n\n",argv[i]);
      dump_open(argv[i]);
    }

  return 0;
}

/********
 * open *
 ********/

static void dump_open(const char *file)
{
  ELEMENT	*e;
  DICTIONARY	*d;

  dicom_log(INFO,"dump_open()");

  if (dicom_open(file))
    return;

  for (;;)
  {
    e=dicom_element();
    if (!e)
      return;

    d=dicom_query(e);

    if (e->vr==UN)
      e->vr=d->vr;

    if (dicom_load(e->vr))
      return;

    dump_print(e,d->description);

    free(e->value.UN);
  }
}

/*********
 * print *
 *********/

static void dump_print(const ELEMENT *e,const char *description)
{
  U32 i;

  dicom_log(INFO,"dump_print()");

  for (i=e->sequence; i; i--)
    printf("  ");

  if (verbose)
    printf("(%.4X,%.4X) %c%c[%u] ",e->group,e->element,e->vr>>8,e->vr&0xFF,
    e->vm);

  printf("%s%s: ",e->encapsulated?"Encapsulated ":"",description);

  if (!e->vm)
  {
    puts("(no value)");
    return;
  }

  if (e->length==0xFFFFFFFF)
  {
    puts("(undefined length)");
    return;
  }

  for (i=0; i<e->vm; i++)
    switch(e->vr)
    {
    case US :
      printf("%u ",e->value.US[i]);
      break;

    case SS :
      printf("%d ",e->value.SS[i]);
      break;

    case UL :
      printf("%u ",e->value.UL[i]);
      break;

    case SL :
      printf("%d ",e->value.SL[i]);
      break;

    case AT :
      printf("(%.4X,%.4X) ",e->value.AT[i].group,e->value.AT[i].element);
      break;

    case FL :
      printf("%f ",e->value.FL[i]);
      break;

    case FD :
      printf("%f ",e->value.FD[i]);
      break;

    case LT :
    case ST :
      printf("[%s] ",e->value.LT);
      break;

    case AE :
    case AS :
    case CS :
    case DA :
    case DS :
    case DT :
    case IS :
    case LO :
    case PN :
    case SH :
    case TM :
    case UI :
      printf("[%s] ",e->value.AE[i]);
      break;

    default :
      printf("(%u bytes)\n",e->length);
      return;
    }

  if (verbose)
    printf("(%u bytes)",e->length);

  puts("");
}
