/****************************
 * dicom image by Tony Voet *
 ****************************/

#include <stdio.h>
#include <dicom.h>

static void image_info(const SINGLE *);

/********
 * main *
 ********/

int main(int argc,char *argv[])
{
  SINGLE	*single;
  int		i;

  if (argc<2)
  {
    fprintf(stderr,"%s (%s)\n",*argv,dicom_version);
    fprintf(stderr,"Usage: %s dicom-file ...\n",*argv);

    return -1;
  }

  dicom_log_name(*argv);

  for (i=1; i<argc; i++)
  {
    puts(argv[i]);

    if (dicom_open(argv[i]))
      continue;

    for (;;)
    {
      single=dicom_single();
      if (!single)
        break;
      image_info(single);
      dicom_single_free();
    }
  }

  return 0;
}

/********
 * info *
 ********/

static void image_info(const SINGLE *single)
{
  U32	i;
  char	*interpretation[]=
  {
    "Monochrome 2",
    "Monochrome 1",
    "Palette color",
    "RGB",
    "HSV",
    "ARGB",
    "CMYK",
    "Unknown"
  },
  *color[]=
  {
    "Red  ",
    "Green",
    "Blue "
  };

  dicom_log(INFO,"image_info()");

  printf("  %u x %u x %u %s",
    single->frames,
    single->w,
    single->h,
    interpretation[single->photometric]);

  if (single->samples>1)
    if (single->planar)
      printf(" (planar)");

  puts("");

  printf("  %u x %u bit/pixel (%u allocated, %u high, %s)\n",
    single->samples,
    single->bit,
    single->alloc,
    single->high,
    single->sign?"signed":"unsigned");

  if (single->photometric==PALETTE_COLOR || single->photometric==ARGB)
    for (i=0; i<3; i++)
    {
      printf("  %s %u x %u bit (threshold ",
        color[i],
	single->clut[i].size,
	single->clut[i].bit);

      if (single->sign)
	printf("%d)\n",single->clut[i].threshold.s16);
      else
	printf("%u)\n",single->clut[i].threshold.u16);

      if (!single->clut[i].data.u16)
	dicom_log(WARNING,"PaletteColorLookupTableData missing");
    }

  puts("");
}
