/****************************
 * dicom merge by Tony Voet *
 ****************************/

#include <stdio.h>
#include <stdlib.h>
#include <dicom.h>

/********
 * main *
 ********/

int main(int argc,char *argv[])
{
  IMAGE	*anatomic=0L,*parametric=0L,*merge=0L;
  int	images;
  U16	min=0,max=0xFFFF,saturation=0xFFFF;

  if (argc<4)
  {
    fprintf(stderr,"%s (%s)\n",*argv,dicom_version);
    fprintf(stderr,"Usage: %s anatomic parametric output [min] [max] [saturation]\n",
      *argv);
    fprintf(stderr,"  min: 16 bit range (default %d)\n",min);
    fprintf(stderr,"  max: 16 bit range (default %d)\n",max);
    fprintf(stderr,"  saturation: 16 bit range (default %d)\n",saturation);

    return -1;
  }

  dicom_log_name(*argv);

  if (dicom_read(argv[1],&anatomic,&images,0))
    return -2;

  if (images!=1)
  {
    dicom_log(ERROR,"Unable to merge a multi-image DICOM");
    goto error;
  }

  if (dicom_read(argv[2],&parametric,&images,-1))
    goto error;

  if (images!=1)
  {
    dicom_log(ERROR,"Unable to merge a multi-image DICOM");
    goto error;
  }

  if (argc>4)
    min=atoi(argv[4]);

  if (argc>5)
    max=atoi(argv[5]);

  dicom_voi(parametric,min,max);

  if (argc>6)
    saturation=atoi(argv[6]);

  merge=dicom_merge(anatomic,parametric,saturation);
  if (!merge)
    goto error;

  dicom_free(anatomic,1);
  dicom_free(parametric,1);

  dicom_write_jpeg(argv[3],merge,95);

  dicom_free(merge,1);

  return 0;

error:
  dicom_free(anatomic,1);
  dicom_free(parametric,1);
  dicom_free(merge,1);

  return -3;
}
