/***********************
 * xdicom by Tony Voet *
 ***********************/

#include <stdlib.h>
#include <math.h>
#include "xdicom.h"
#include "bitmap/cross"

/*!!!!!!!!!*/

int image_min,image_max;

Display	*display;
Visual	*visual;
Window	window;

static GC	gc;
static int	cross;

IMAGE *zoom=0L;

/**/

extern unsigned char	*Ir,*Ig,*Ib;
extern int		size,K;

extern unsigned char	lut_r[],lut_g[],lut_b[];
extern unsigned short int	*Qadd;

extern Colormap	colormap;
extern u_long	*pixel;

void cqmain(void);

void patch(void)
{
  U32		l;
  U8		*source;
  unsigned char	*red,*green,*blue;

  XColor	*c;
  int		i;

  void		*data;
  unsigned short int	*s;
  U8		*dc;

  size=zoom->w*zoom->h;

  puts("patch: Splitting RGB components");

  Ir=malloc(size);
  Ig=malloc(size);
  Ib=malloc(size);

  if (!Ir || !Ig || !Ib)
  {
    dicom_log(EMERGENCY,"Out of memory");
    return;
  }

  source=zoom->data.rgb+3*frame*size;

  red=Ir;
  green=Ig;
  blue=Ib;

  for (l=size; l; l--)
  {
    *red++=*source++;
    *green++=*source++;
    *blue++=*source++;
  }

  puts("patch: Calling color quantization");

  K=colors;
  cqmain();

  puts("patch: Filling colormap");

  c=color;
  for (i=0; i<colors; i++)
  {
    printf("%i: %u %u %u\n",i,lut_r[i],lut_g[i],lut_b[i]);
    c->red=0xFFU*lut_r[i];
    c->green=0xFFU*lut_g[i];
    c->blue=0xFFU*lut_b[i];
/*    c->flags=DoRed|DoGreen|DoBlue;
    c->pixel=pixel[i];*/
    c++;
  }

  puts("patch: Storing colormap");

  XStoreColors(display,colormap,color,colors);

  puts("patch: Calculating XImage");

  data=malloc(size);
  if (!data)
  {
    dicom_log(ERROR,"Out of memory");
    return;
  }

  s=Qadd;
  dc=data;

  for (l=size; l; l--)
    *dc++=color[*s++].pixel;

  free(Qadd);

  puts("patch: Creating XImage");

  wc=XCreateImage(display,visual,8,ZPixmap,0,data,zoom->w,zoom->h,
  8,0);
}

/*****************
 * update canvas *
 *****************/

int canvas_update(void)
{
  Window	main;
  int		w,h,ww,hh;

  dicom_log(INFO,"canvas_update()");

  canvas_free();

  w=form_main->main->w;
  h=form_main->main->h;

  ww=sqrt((double) w*h*image->w/image->h)+0.5;
  hh=sqrt((double) w*h*image->h/image->w)+0.5;

  fl_set_form_size(form_main->main,ww,hh);

  main=FL_ObjWin(form_main->background);
  fl_winaspect(main,ww,hh);

  if (w==ww && h==hh)
  {
    fl_get_win_size(window,&w,&h);
    return canvas_expose(w,h);
  }

  return 0;
}

/**********
 * canvas *
 **********/

static int canvas(FL_OBJECT *object,Window window,int w,int h,XEvent *event,
void *data)
{
  int x,y;

  dicom_log(INFO,"canvas()");

  if (!image && event->type!=Expose)
    return 0;

  switch(event->type)
  {

  case Expose :
    menu_resize(w,h);
    canvas_expose(w,h);
    break;

  case ButtonPress :
    XGrabPointer(display,window,True,None,GrabModeAsync,GrabModeAsync,window,
    None,CurrentTime);

    if (event->xbutton.button==1)
      puts("Begin Mode");
    else
      wc_start(w,h);
    break;

  case ButtonRelease :
    XUngrabPointer(display,CurrentTime);

    if (event->xbutton.button==1)
      puts("End Mode");
    else
      wc_end(w,h,cross);
    break;

  case MotionNotify :
    x=event->xmotion.x;
    y=event->xmotion.y;

    if (event->xmotion.state & 0x100)
      printf("Mode (%d,%d)\n",x,y);
    else
      wc_set(x,y,w,h);
    break;

  default :
    dicom_log(WARNING,"Unwanted X-event");
  }

  return 0;
}

/***************
 * init canvas *
 ***************/

void canvas_init(void)
{
  XWindowAttributes	attribute;
  XGCValues		*value=0L;

  dicom_log(INFO,"canvas_init()");

/*!!!!!!*/
  image_min=0;
  image_max=0xFFFF;

  display=fl_get_display();

  window=FL_ObjWin(form_main->canvas);
  XGetWindowAttributes(display,window,&attribute);

  visual=attribute.visual;

  gc=DefaultGCOfScreen(attribute.screen);
  XGetGCValues(display,gc,0,value);
  XFreeGC(display,gc);
  gc=XCreateGC(display,window,0,value);

  cross=fl_create_bitmap_cursor((char*) cross_bits,(char*) cross_bits,
  cross_width,cross_height,cross_x_hot,cross_y_hot);
  fl_set_cursor_color(cross,FL_RED,FL_BLACK);

  fl_set_cursor(window,cross);

  fl_add_canvas_handler(form_main->canvas,Expose,canvas,0L);

  fl_add_canvas_handler(form_main->canvas,ButtonPress,canvas,0L);
  fl_add_canvas_handler(form_main->canvas,ButtonRelease,canvas,0L);

  fl_add_canvas_handler(form_main->canvas,MotionNotify,canvas,0L);
  fl_remove_selected_xevent(window,PointerMotionMask|PointerMotionHintMask);

  zoom=0L;
  wc_init();
  colormap_init();
}

/*****************
 * expose canvas *
 *****************/

int canvas_expose(int w,int h)
{
  static int lw,lh,lmin,lmax;

  dicom_log(INFO,"canvas_expose()");

/***/
  printf("Expose %d x %d\n",w,h);
/***/

  if (!zoom || w!=lw || h!=lh)
  {
    canvas_free();
    zoom=dicom_zoom(image,w,h,0);
    if (!zoom)
      return -1;
  }

  if (!wc || image_min!=lmin || image_max!=lmax)
    if (image->rgb)
      patch();
    else
      wc_change();

  XPutImage(display,window,gc,wc,0,0,0,0,w,h);

  lw=w;
  lh=h;
  lmin=image_min;
  lmax=image_max;

  return 0;
}

/***************
 * free canvas *
 ***************/

void canvas_free(void)
{
  dicom_log(INFO,"canvas_free()");

  wc_free();

  dicom_free(zoom,1);
  zoom=0L;

/*
  XFreeGC(display,gc);
*/
}
