/***********************
 * xdicom by Tony Voet *
 ***********************/

#include <stdlib.h>
#include "xdicom.h"

XColor	*color=0L;
int	colors,rwcolor=0;

Colormap	colormap;
u_long	*pixel=0L;

/*****************
 * init colormap *
 *****************/

int colormap_init(void)
{
  XColor	*c;
  int		i;

  dicom_log(INFO,"colormap_init()");

  if (visual->class==DirectColor)
  {
    dicom_log(CRITICAL,"DirectColor visual class is not supported");
    xdicom_free();
    exit(-1);
  }

  if (visual->class & 1)
    rwcolor=-1;

  colormap=fl_get_canvas_colormap(form_main->canvas);

  colors=visual->map_entries;
  if (colors>256)
    colors=256;

  pixel=malloc(colors*sizeof(u_long));
  if (!pixel)
    goto memory;

  if (rwcolor)
  {
    for (;; colors--)
    {
      if (XAllocColorCells(display,colormap,False,0L,0,pixel,colors))
        break;
    }

    pixel=realloc(pixel,colors*sizeof(u_long));
    if (!pixel)
      goto memory;
    /* free pixel! */
  }

  color=malloc(colors*sizeof(XColor));
  if (!color)
    goto memory;

  c=color;
  for (i=0; i<colors; i++)
  {
    c->red=65535*i/(colors-1);
    c->green=c->red;
    c->blue=c->red;
    c->flags=DoRed|DoGreen|DoBlue;

    if (rwcolor)
      c->pixel=pixel[i];
    else
    {
      XAllocColor(display,colormap,c);
      pixel[i]=c->pixel;
    }

    c++;
  }

  if (rwcolor)
    XStoreColors(display,colormap,color,colors);

/****/
  printf("%d colors allocated\n",colors);
/****/

  return 0;

memory:
  dicom_log(ERROR,"Out of memory");

  free(pixel);
  pixel=0L;
  free(color);
  color=0L;

  return -1;
}

/*******************
 * change colormap *
 *******************/

void colormap_change(void)
{
  XColor	*c;
  int		min,max,i;

  dicom_log(INFO,"colormap_change()");

  if (!rwcolor)
    return;

  min=(colors-1)*image_min/0xFFFF;
  max=(colors-1)*image_max/0xFFFF;

  printf("Colormap min %d max %d\n",min,max);

  c=color;
  for (i=0; i<colors; i++,c++)
  {
    if (i<=min)
    {
      c->red=0;
      continue;
    }
    if (i>=max)
    {
      c->red=65535;
      continue;
    }
    c->red=(i-min)*65536/(max-min);
  }

  c=color;
  for (i=colors; i; i--)
  {
    c->green=c->red;
    c->blue=c->red;
    c++;
  }

  XStoreColors(display,colormap,color,colors);
}

/******************
 * reset colormap *
 ******************/

void colormap_reset(void)
{
  XColor	*c;
  int		i;

  dicom_log(INFO,"colormap_reset()");

  if (!rwcolor)
    return;

  c=color;
  for (i=0; i<colors; i++)
  {
    c->red=65535*i/(colors-1);
    c->green=c->red;
    c->blue=c->red;
    c++;
  }

  XStoreColors(display,colormap,color,colors);
}

/*****************
 * free colormap *
 *****************/

void colormap_free(void)
{
  dicom_log(DEBUG,"colormap_free()");

  if (!color)
    return;

  XFreeColors(display,colormap,pixel,colors,0);

  free(pixel);
  free(color);

  color=0L;
}
