/***********************
 * xdicom by Tony Voet *
 ***********************/

#include <stdlib.h>
#include "xdicom.h"

static char **menu_parse(const char *,FL_OBJECT *);

#define LINE 16384

#define OPEN	1
#define SAVE	2
#define SAVE_AS	3
#define CLOSE	4

#define EXPORT_ASCII	5
#define EXPORT_JPEG	6
#define EXPORT_EPS	7

#define PRINT	8
#define QUIT	9

#define IMAGE	1
#define STUDY	2
#define PATIENT	3

#define ABOUT	1
#define HELP	2

static FD_about	*form_about;
static int	scope=IMAGE;
static char	**shell=0L,**transfer=0L;

/********
 * init *
 ********/

void menu_init(void)
{
  dicom_log(INFO,"menu_init()");

  menu_free();

  fl_set_menu(form_main->file,"Open|Save%d|Save As%d|Close%d%l|Export ASCII%d|Export JPEG%d|Export EPS%d%l|Print%d%l|Quit");

  fl_set_menu_item_shortcut(form_main->file,OPEN,"Oo#O#o");
  fl_set_menu_item_shortcut(form_main->file,SAVE,"Ss#S#s");
  fl_set_menu_item_shortcut(form_main->file,SAVE_AS,"Aa#A#a");
  fl_set_menu_item_shortcut(form_main->file,CLOSE,"Cc#C#c");

  fl_set_menu_item_shortcut(form_main->file,EXPORT_ASCII,"Ii#I#i");
  fl_set_menu_item_shortcut(form_main->file,EXPORT_JPEG,"Jp#J#j");
  fl_set_menu_item_shortcut(form_main->file,EXPORT_EPS,"Ee#E#e");

  fl_set_menu_item_shortcut(form_main->file,PRINT,"Pp#P#p");

  fl_set_menu_item_shortcut(form_main->file,QUIT,"Qq#Q#q");

  fl_set_menu(form_main->scope,"Image%R1|Study%r1|Patient%r1");

  fl_set_menu_item_shortcut(form_main->scope,IMAGE,"Ii#I#i");
  fl_set_menu_item_shortcut(form_main->scope,STUDY,"Ss#S#s");
  fl_set_menu_item_shortcut(form_main->scope,PATIENT,"Pp#P#p");

  shell=menu_parse("/home/tvoet/.xdicom/shell",form_main->shell);
  transfer=menu_parse("/home/tvoet/.xdicom/transfer",form_main->transfer);

  fl_set_menu(form_main->help,"About Xrad%l|Help%d");

  fl_set_menu_item_shortcut(form_main->help,ABOUT,"Aa#A#a");
  fl_set_menu_item_shortcut(form_main->help,HELP,"Hh#H#h");

  form_about=create_form_about();
}

/********
 * file *
 ********/

void menu_file(FL_OBJECT *object,long data)
{
  dicom_log(INFO,"menu_file()");

  switch(fl_get_menu(object))
  {
  case OPEN :
    fl_set_fselector_title("Open");

    if (fl_show_fselector("Open image","","*",""))
    {
      directory_set(fl_get_directory(),(char*) fl_get_filename());
      slider_open(-1);
    }
    break;

  case QUIT :
    xdicom_free();
    exit(0);
  }
}

/*********
 * scope *
 *********/

void menu_scope(FL_OBJECT *object,long data)
{
  dicom_log(INFO,"menu_scope()");

  scope=fl_get_menu(object);
}

/*********
 * shell *
 *********/

void menu_shell(FL_OBJECT *object,long data)
{
  char command[LINE];

  dicom_log(INFO,"menu_shell()");

  switch(scope)
  {
  case IMAGE :
    sprintf(command,shell[fl_get_menu(object)-1],directory[current]);
    break;

  case STUDY :
    sprintf(command,shell[fl_get_menu(object)-1],"*");
    break;

  case PATIENT :
    sprintf(command,shell[fl_get_menu(object)-1],"../*/*");
  }

  fl_show_command_log(FL_FULLBORDER);
  fl_exe_command(command,0);
}

/************
 * transfer *
 ************/

void menu_transfer(FL_OBJECT *object,long data)
{
  dicom_log(INFO,"menu_transfer()");
}

/********
 * help *
 ********/

void menu_help(FL_OBJECT *object,long data)
{
  dicom_log(INFO,"menu_help()");

  switch(fl_get_menu(object))
  {
  case ABOUT :
    fl_show_form(form_about->about,FL_PLACE_CENTER,FL_TRANSIENT,"About Xdicom");
  }
}

/********
 * free *
 ********/

void menu_free(void)
{
  int	i;
  char	**item;

  dicom_log(INFO,"menu_free()");

  if (shell)
  {
    i=fl_get_menu_maxitems(form_main->shell);
    item=shell;

    for (; i; i--)
      free(*item++);

    free(shell);
  }

  if (transfer)
  {
    i=fl_get_menu_maxitems(form_main->transfer);
    item=transfer;

    for (; i; i--)
      free(*item++);

    free(transfer);
  }
}

/*********
 * parse *
 *********/

static char **menu_parse(const char *file,FL_OBJECT *object)
{
  FILE	*stream;
  int	entry;
  char	line[LINE],*c,
	menu[LINE]="",
	**command,**tmp;

  dicom_log(INFO,"menu_parse()");

  stream=fopen(file,"r");
  if (!stream)
    return 0L;

  for (command=0L,entry=0;;)
  {
    fgets(line,LINE,stream);

    if (feof(stream))
      break;

    if (*line=='\n')
    {
      if (*menu)
        strcat(menu,"%l");
      continue;
    }

    for (c=line; *c!='\t'; c++);
    *c=0;

    if (*menu)
      strcat(menu,"|");
    strcat(menu,line);

    for (c++; *c=='\t'; c++);
    c[strlen(c)-1]=0;

    tmp=realloc(command,(entry+1)*sizeof(char*));
    if (!tmp)
    {
      dicom_log(ERROR,"Out of memory");
      continue;
    }
    command=tmp;

    command[entry]=malloc((strlen(c)+1)*sizeof(char));
    if (!command[entry])
    {
      dicom_log(ERROR,"Out of memory");
      continue;
    }

    strcpy(command[entry],c);
    entry++;
  }

  fclose(stream);

  fl_set_menu(object,menu);

  return command;
}

/**********
 * resize *
 **********/

void menu_resize(int w,int h)
{
  static int lw=-1,lh=-1,pt[]=
  {
    FL_HUGE_SIZE,
    FL_LARGE_SIZE,
    FL_MEDIUM_SIZE,
    FL_NORMAL_SIZE,
    FL_SMALL_SIZE,
    FL_TINY_SIZE
  };

  int size,style,i;

  dicom_log(INFO,"menu_resize()");

  if (w==lw && h==lh)
    return;

  size=FL_TINY_SIZE;
  for (i=0; i<6; i++)
    if (pt[i]<0.72*(form_main->file->h-4))
    {
      size=pt[i];
      break;
    }

  if (size>FL_SMALL_SIZE)
    style=FL_BOLD_STYLE;
  else
    style=FL_NORMAL_STYLE;

  fl_freeze_form(form_main->main);

  fl_set_object_lsize(form_main->file,size);
  fl_set_object_lsize(form_main->scope,size);
  fl_set_object_lsize(form_main->shell,size);
  fl_set_object_lsize(form_main->transfer,size);
  fl_set_object_lsize(form_main->help,size);

  fl_setpup_default_fontsize(size);

  fl_set_object_lstyle(form_main->file,style);
  fl_set_object_lstyle(form_main->scope,style);
  fl_set_object_lstyle(form_main->shell,style);
  fl_set_object_lstyle(form_main->transfer,style);
  fl_set_object_lstyle(form_main->help,style);

  fl_setpup_default_fontstyle(style);

  fl_redraw_form(form_main->main);
  fl_unfreeze_form(form_main->main);

  lw=w;
  lh=h;
}
