/***********************
 * xdicom by Tony Voet *
 ***********************/

#include "xdicom.h"
#include "bitmap/xdicom"

static void slider_title(void);

IMAGE	*image=&xdicom;
int	frame=0;

static IMAGE	*dicom=0L;
static int	images,position,size;
static enum
{
  UNSET,
  CLEAR,
  DIRECTORY,
  MULTI
}
status=UNSET;

/****************
 * clear slider *
 ****************/

void slider_clear(void)
{
  dicom_log(INFO,"slider_clear()");

  if (status==CLEAR)
    return;

  status=CLEAR;

  slider_title();
  fl_set_slider_size(form_main->slider,1.0);
  fl_deactivate_object(form_main->slider);

  directory_free();
  slider_free();

  canvas_update();
}

/***************
 * slider open *
 ***************/

int slider_open(int cd)
{
  int error,update=-1,i;

  dicom_log(INFO,"slider_open()");

  if (current<0)
    return -1;

  slider_free();

  error=dicom_read(directory[current],&dicom,&images,0);

  if (!error)
  {
    size=0;
    image=dicom;
    for (i=images; i; i--)
      size+=image++->frames;

    image=dicom;
    frame=0;
  }

  if (error || size==1)
  {
    size=entries;
    position=current;

    if (status==DIRECTORY)
      update=cd;
    else
      status=DIRECTORY;
  }
  else
  {
    position=0;
    status=MULTI;
  }

  slider_title();

  if (update)
  {
    fl_set_slider_bounds(form_main->slider,0.0,size-1);
    fl_set_slider_value(form_main->slider,position);
    fl_set_slider_size(form_main->slider,1.0/size);

    fl_set_slider_step(form_main->slider,1.0);
    fl_set_slider_increment(form_main->slider,1.0,size);

    fl_activate_object(form_main->slider);
  }

  canvas_update();

  return 0;
}

/***************
 * free slider *
 ***************/

void slider_free(void)
{
  dicom_log(INFO,"slider_free()");

  if (!dicom)
    return;

  dicom_free(dicom,images);
  dicom=0L;

  image=&xdicom;
  frame=0;
}

/**********
 * slider *
 **********/

void slider(FL_OBJECT *object,long data)
{
  int i;

  dicom_log(INFO,"slider()");

  position=fl_get_slider_value(object);

  switch(status)
  {
  case DIRECTORY :
    current=position;
    slider_open(0);
    break;

  case MULTI :
    slider_title();

    image=dicom;
    frame=position;

    for (i=images; i; i--,image++)
      if (frame>=image->frames)
	frame-=image->frames;
      else
	break;

    canvas_update();
    break;

  default:
    dicom_log(WARNING,"Slider unset or cleared");
  }
}

/****************
 * slider title *
 ****************/

static void slider_title(void)
{
  static char title[180];

  dicom_log(INFO,"slider_title()");

  switch(status)
  {
  case UNSET :
  case CLEAR :
    fl_set_form_title(form_main->main,"Xdicom");
    return;

  case DIRECTORY :
    sprintf(title,"Xdicom - %.128s",directory[current]);
    break;

  case MULTI :
    sprintf(title,"Xdicom - %.128s %d/%d",directory[current],position+1,size);
    break;
  }

  fl_set_form_title(form_main->main,title);
}
