/***********************
 * xdicom by Tony Voet *
 ***********************/

#include <stdlib.h>
#include "xdicom.h"
#include "bitmap/ramp"

XImage *wc=0L;

static FD_wc	*form_wc;
static int	depth,ramp;

/***********
 * init wc *
 ***********/

void wc_init(void)
{
  dicom_log(DEBUG,"wc_init()");

  depth=visual->bits_per_rgb;

  if (depth!=8 && depth!=16 && depth!=24 && depth!=32)
  {
    dicom_log(CRITICAL,"Unsupported depth");
    xdicom_free();
    exit(-1);
  }

  form_wc=create_form_wc();

  ramp=fl_create_bitmap_cursor((char*) ramp_bits,(char*) ramp_bits,
  ramp_width,ramp_height,ramp_x_hot,ramp_y_hot);
  fl_set_cursor_color(ramp,FL_GREEN,FL_BLACK);
}

/*************
 * change wc *
 *************/

int wc_change(void)
{
  int		min,max,i;
  void		*data;
  U32	*dl;
  U16	*ds,*s;
  U8	*dc;

  dicom_log(DEBUG,"wc_change()");

  wc_free();

  min=image_min;
  max=image_max;

/**/  data=malloc(3*zoom->w*zoom->h*depth>>3);
  if (!data)
  {
    dicom_log(ERROR,"Out of memory");
    return -1;
  }

  s=zoom->data.gray+frame*zoom->w*zoom->h;

  switch(depth)
  {
  case 8 :
    dc=data;
    for (i=zoom->w*zoom->h; i; i--,s++)
    {
      if (*s<=min)
      {
	*dc++=color[0].pixel;
	continue;
      }
      if (*s>=max)
      {
	*dc++=color[colors-1].pixel;
	continue;
      }
      *dc++=color[(*s-min)*(colors-1)/(max-min)].pixel;
    }
    break;

  case 16 :
    ds=data;
    for (i=zoom->w*zoom->h; i; i--,s++)
    {
      if (*s<=min)
      {
	*ds++=color[0].pixel;
	continue;
      }
      if (*s>=max)
      {
	*ds++=color[colors-1].pixel;
	continue;
      }
      *ds++=color[(*s-min)*(colors-1)/(max-min)].pixel;
    }
    break;

  case 24 :
  case 32 :
    dl=data;
    for (i=zoom->w*zoom->h; i; i--,s++)
    {
      if (*s<=min)
      {
	*dl++=color[0].pixel;
	continue;
      }
      if (*s>=max)
      {
	*dl++=color[colors-1].pixel;
	continue;
      }
      *dl++=color[(*s-min)*(colors-1)/(max-min)].pixel;
    }
  }

  printf("Visual %d - Depth %d\n",visual->class,depth);

  wc=XCreateImage(display,visual,depth,ZPixmap,0,data,zoom->w,zoom->h,depth,0);

  puts("here");
  
  return 0;
}

/***********
 * free wc *
 ***********/

void wc_free(void)
{
  dicom_log(DEBUG,"wc_free()");

  if (!wc)
    return;

  XDestroyImage(wc);

  wc=0L;
}

/************
 * start wc *
 ************/

void wc_start(int w,int h)
{
  int	x,y,win,cen;
  char	label[32];

  dicom_log(DEBUG,"wc_start()");

  win=image_max-image_min+1;
  cen=(image_min+image_max+1)>>1;

  x=win*(w-1)/0x10000;
  y=cen*(h-1)/0xFFFF;

/******/
  printf("min %d max %d -> W %d C %d\n",image_min,image_max,win,cen);
/******/

  XWarpPointer(display,None,window,0,0,0,0,x,y);

  fl_set_cursor(window,ramp);

  fl_get_winorigin(window,&x,&y);
  x+=(w-form_wc->label->w)>>1;
  y+=(h-form_wc->label->h)>>1;
  fl_set_form_position(form_wc->wc,x,y);

  sprintf(label,"W %d\nC %d",win,cen);
  fl_set_object_label(form_wc->label,label);

  fl_show_form(form_wc->wc,FL_PLACE_POSITION,FL_NOBORDER,"Xrad WC");

  if (rwcolor)
  {
/*    canvas_clear(w,h);*/
    colormap_change();
    x=image_min;
    y=image_max;
    image_min=0;
    image_max=0xFFFF;
    canvas_expose(w,h);
    image_min=x;
    image_max=y;
  }
}

/**********
 * set wc *
 **********/

void wc_set(int x,int y,int w,int h)
{
  int	win,cen;
  char	label[32];

  dicom_log(DEBUG,"wc_set()");

  win=0x10000*x/(w-1);
  cen=0xFFFF*y/(h-1);

  image_min=cen-(win>>1);
  image_max=cen+(win>>1)-1;

  if (win<2)
    image_max=image_min;

  if (image_min<0)
    image_min=0;

  if (image_max>0xFFFF)
    image_max=0xFFFF;

/******/
  printf("W %d C %d -> min %d max %d\n",win,cen,image_min,image_max);
/******/

  sprintf(label,"W %d\nC %d",image_max-image_min+1,(image_min+image_max+1)>>1);

  fl_set_object_label(form_wc->label,label);

  if (rwcolor)
    colormap_change();
  else
    canvas_expose(w,h);
}

/**********
 * end wc *
 **********/

void wc_end(int w,int h,int cursor)
{
  dicom_log(DEBUG,"wc_end()");

  fl_set_cursor(window,cursor);

  fl_hide_form(form_wc->wc);

  if (rwcolor)
  {
/*    canvas_clear(w,h);*/
    colormap_reset();
    canvas_expose(w,h);
  }
}

/***********
 * voi set *
 ***********

int voi_set(int min,int max)
{
  U32	length,l;
  U16	*gray;
  U8	*rgb;

  dicom_log(INFO,"voi_set()");

  if (zoom->rgb)
  {
    length=zoom->w*zoom->h;
    rgb=zoom->data.rgb+frame*length*3;

    for (l=length; l; l--)
      =*rgb++;
  }
  else
    gray=zoom->data.gray+frame*zoom->w*zoom->h;

  switch(visual->class)
  {
  case StaticGray :
  case GrayScale :
  case StaticColor :
  case PseudoColor :
  case TrueColor :
  case DirectColor :
  }

  printf("Visual %d - depth %d\n",visual->class,depth);

  wc=XCreateImage(display,visual,depth,ZPixmap,0,data,zoom->w,zoom->h,depth,0);
}
*/
